/*************************************************************/
/* iTraceroute (part of iSPY) Project                        */
/* Copyright (c) 2008, Zheng Zhang, Purdue University        */
/*************************************************************/

#ifndef __COMMON_H__
#define __COMMON_H__

//#define __OS_LINUX__

#include <sys/time.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>

/* Some basic type definitions */
typedef unsigned int    uint32;
typedef signed   int    sint32;
typedef unsigned short  uint16;
typedef signed   short  sint16;
typedef unsigned char   uint8;
typedef signed   char   sint8;

// Debug stuffs

// Log levels
#define FATAL   0                                 // Log "fatal error" messages, abord
#define ERROR   1                                 // Log "error" messages, continue
#define WARN    2                                 // Log "warning" and "todo" messages
#define INFO    3                                 // Log "info" messages	( -quiet )
#define DUMP    4                                 // Log "debug" messages	( -v )

void set_log_level (int lvl);
int  get_log_level ();

#define STR_FATAL   "FATAL"
#define STR_ERROR   "ERROR"
#define STR_WARN    "WARN"
#define STR_INFO    "INFO"
#define STR_DUMP    "DEBUG"
#define STR_UNKNOW  "UNKNOW"
const char* lvl2str (int lvl);

/* Log functions
 *
 * lvl  : severity level of the message
 * args : message in a 'printf' friendly form
 */
#ifdef __GNUC__
void log_c (int lvl, const char* file, int line, const char* format, ...)
__attribute__ ((format (printf, 4, 5)));
#define log(lvl, args... ) \
if (lvl <= get_log_level()) log_c(lvl, __FILE__, __LINE__, ## args )

const char* str_log_c (int lvl, const char* file, int line,
const char* format, ...) __attribute__ ((format (printf, 4, 5)));
#define str_log(lvl, args...) str_log_c(lvl, __FILE__, __LINE__, ## args )

#else                                             // __GNUC__
void log_c (int lvl, const char* file, int line, const char* format, ...);
#define log(lvl, ...) \
if (lvl <= get_log_level()) log_c(lvl, __FILE__, __LINE__, __VA_ARGS__)
const char* str_log_c (int lvl, const char* file, int line,
const char* fortmat, ...);
#define str_log(lvl, ...) str_log_c(lvl, __FILE__, __LINE__, __VA_ARGS__)
#endif                                            // __GNUC__

void dumpRawData (int lvl, uint8* data, int len);

void ASSERT(int cond, const char * errformat, ...);

//Exception
class MyException
{
private:
    char*   reason;
    
public:
    MyException(const char* r) {
        fprintf (stderr, "Exception : %s\n", r);
        fflush (stderr);
        reason = strdup(r);
    }
    
    ~MyException() {
        //free(reason);  //Zheng, this free triggers "double free" and then seg fault ???
    }
    
    char* getReason () {
        return reason;
    }
};

//#define timeval_diff(t1, t2) (((t1)->tv_sec-(t2)->tv_sec)*1000000+(t1)->tv_usec-(t2)->tv_usec)
int timeval_diff(const struct timeval& t1, const struct timeval& t2);
struct timeval timeval_incr(const struct timeval& t, int incr_usec);

//void setup_sigsegv(void);
void get_mytime(struct timeval* tm);

#endif                                            // __COMMON_H__
