/*************************************************************/
/* iTraceroute (part of iSPY) Project                        */
/* Copyright (c) 2008, Zheng Zhang, Purdue University        */
/*************************************************************/

#ifndef __NET_UTIL_H__
#define __NET_UTIL_H__

#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/in.h>

#include <string>
using namespace std;

#include "Common.h"

#if !defined(IPVERSION)
#define IPVERSION 4  /* Incase some system does not have this definition. */
#endif 

#if defined(X2_OFF)         /* SCO */
#define TH_OFFSET 0x50         /* TCP header offset */
#else
#define TH_OFFSET 5
#endif

#define DEFAULT_TTL 128  /* Just hard code the ttl in the ip header.*/
#define TCP_WINDOW_SIZE 512 /* Just have it hardcoded. */

struct _iphdr
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
    unsigned int ihl:4;
    unsigned int version:4;
#elif __BYTE_ORDER == __BIG_ENDIAN
    unsigned int version:4;
    unsigned int ihl:4;
#else
#error "Please fix <bits/endian.h>"
#endif
    u_int8_t tos;
    u_int16_t tot_len;
    u_int16_t id;
    u_int16_t frag_off;
    u_int8_t ttl;
    u_int8_t protocol;
    u_int16_t check;
    u_int32_t saddr;
    u_int32_t daddr;
};

struct _tcphdr
{
    u_int16_t source;
    u_int16_t dest;
    u_int32_t seq;
    u_int32_t ack_seq;
#  if __BYTE_ORDER == __LITTLE_ENDIAN
    u_int16_t res1:4;
    u_int16_t doff:4;
    u_int16_t fin:1;
    u_int16_t syn:1;
    u_int16_t rst:1;
    u_int16_t psh:1;
    u_int16_t ack:1;
    u_int16_t urg:1;
    u_int16_t res2:2;
#  elif __BYTE_ORDER == __BIG_ENDIAN
    u_int16_t doff:4;
    u_int16_t res1:4;
    u_int16_t res2:2;
    u_int16_t urg:1;
    u_int16_t ack:1;
    u_int16_t psh:1;
    u_int16_t rst:1;
    u_int16_t syn:1;
    u_int16_t fin:1;
#  else
#   error "Adjust your <bits/endian.h> defines"
#  endif
    u_int16_t window;
    u_int16_t check;
    u_int16_t urg_ptr;
};

struct _udphdr
{
    u_int16_t source;
    u_int16_t dest;
    u_int16_t len;
    u_int16_t check;
};

#define _ICMP_TIME_EXCEEDED      11      /* Time Exceeded                */
#define _ICMP_DEST_UNREACH       3       /* Destination Unreachable      */
#define _ICMP_TYPE_ECHO_REQUEST 8
#define _ICMP_TYPE_ECHO_REPLY 0

struct _icmphdr
{
  u_int8_t type;                /* message type */
  u_int8_t code;                /* type sub-code */
  u_int16_t checksum;
  union
  {
    struct
    {
      u_int16_t id;
      u_int16_t sequence;
    } echo;                     /* echo datagram */
    u_int32_t   gateway;        /* gateway address */
    struct
    {
      u_int16_t __unused;
      u_int16_t mtu;
    } frag;                     /* path mtu discovery */
  } un;
};


class NetUtil {
public:
    static string ip_uint32_to_str(uint32 addr);
    static void ip_gen(char *packet, unsigned char protocol, uint32 saddr, uint32 daddr, unsigned short ip_id, unsigned short length, uint8 ttl);
    static void tcp_gen(char *packet, uint16 sport, uint16 dport, uint32 seq, uint32 ack, char f_syn, char f_ack, char f_rst);
    static string print_ip(struct _iphdr *p);
    static string print_tcp(struct _tcphdr *p);
    static string print_icmp(struct _icmphdr *p);
    static unsigned short in_cksum2(unsigned short *addr, int len);
    static unsigned short trans_check(unsigned char proto, char *packet, int length, uint32 saddr, uint32 daddr);
    static uint32 getRoute (uint32 destip);
    static void send_tcp_packet (int sockd, uint32 saddr, uint32 daddr, uint16 sport, uint16 dport, 
                                 uint32 seq, uint32 ack, char f_syn, char f_ack, char f_rst, uint8 ttl = DEFAULT_TTL); //char* payload = NULL, int payload_len = 0);
    static void send_udp_packet (int sockd, uint32 saddr, uint32 daddr, uint16 sport, uint16 dport,
                                 uint16 ip_id, uint8 ttl = DEFAULT_TTL); //char* payload, int payload_len);
    static void send_icmp_echoreq_packet (int sockd, uint32 saddr, uint32 daddr, uint16 icmp_id, uint16 icmp_seq, uint8 ttl = DEFAULT_TTL);
};


#endif //__NET_UTIL_H__
