#ifndef __PA_TRACEROUTE_H__
#define __PA_TRACEROUTE_H__

#include <string>
#include <vector>
//#include <ext/hash_map>
using namespace std;
//using namespace __gnu_cxx;

#include "Common.h"
#include "PacketCatcher.h"
#include "patricia.h"

struct TTL_Reply_Detail {
    enum Logic_Code {
        ON_THE_WAY,
        DEST_REACHED,
        UNEXPECTED
    };

    uint32 from_ip;
    uint8 ttl;
    Logic_Code logic_code;
    string net_code;
};

//class PA_Traceroute has no idea of how probe packet is crafted and how to parse probe echo
//class ProbePacket takes care of this dirty work
//class ProbePacket should be able to support
// .send
// .parse
//class PA_Traceroute relies on PacketCatcher to receive raw packet, it is a subscriber to PacketCatcher

template <class ProbePacket>
class PA_Traceroute
{
public:
    PA_Traceroute (IncomPktMgr_Client* pkt_clnt, ProbePacket* probep, 
        unsigned short init_pkt_id, unsigned short max_pkt_id = 0xffff);
    ~PA_Traceroute ();

    void addOption_StartTTL (int start_ttl);
    void addOption_MaxTTL (int max_ttl);
    void addOption_Timeout (int timeout_msec);
    void addOption_nProbe_per_Hop (int nprobe_per_hop);
    void addOption_nMaxGoodProbe_per_Hop (int nMaxGoodProbe_per_hop);
    void addOption_Enter_Stop_AS (int stop_AS);
    void addOption_Consecu_Noresp_Abort (int nhops_abort);
    void addOption_IP2AS_Mapping (patricia_tree_t* ip2AS_mapping);
    
    void run (void);
    string output (void);
    
protected:
    int wait_for_reply (int timeout_msec, uint16 expected_seq, struct timeval *tm_recv,
        TTL_Reply_Detail *detail);

    bool check_stop_max_ttl (string* reason_to_stop, uint8 ttl);
    bool check_stop_enter_AS (string* reason_to_stop, int ASN);
    bool check_dest_reached (string* reason_to_stop, struct TTL_Reply_Detail* reply_detail);
    bool check_stop_consecu_noresp (string* reason_to_stop, int consecutive_noresp);
    
    ProbePacket* probep_;
    
    struct Per_Probe_Record
    {
        unsigned int from_ip;
        double rtt_msec;
        string return_code;
        struct timeval tm_sent;
        int ASN;
    };
    
    struct Hop_Record
    {
        vector<Per_Probe_Record> tries;        
    };

    void post_process_reply (struct Per_Probe_Record* cur_ent);
    
    vector<Hop_Record> records_;

    int nprobe_per_hop_, nMaxGoodProbe_per_hop_, nhops_abort_;
    uint8 start_ttl_, max_ttl_;
    int timeout_msec_;    
    patricia_tree_t* ip2AS_mapping_;
    int stop_enter_AS;
    
    IncomPktMgr_Client* pkt_clnt_;
    unsigned short init_pkt_id_, max_pkt_id_;

    string reason_to_stop;
};


#include <iostream>
#include <sstream>
using namespace std;
#include <sys/time.h>
#include <pthread.h>
#include <errno.h>

#include "Common.h"
#include "NetUtil.h"


template <class ProbePacket>
PA_Traceroute<ProbePacket>::
PA_Traceroute (IncomPktMgr_Client* pkt_clnt, ProbePacket* probep, 
        unsigned short init_pkt_id, unsigned short max_pkt_id)
{
    pkt_clnt_ = pkt_clnt;
    probep_ = probep;
    init_pkt_id_ = init_pkt_id;
    max_pkt_id_ = max_pkt_id;
    nprobe_per_hop_ = 3;
    nMaxGoodProbe_per_hop_ = -1;
    timeout_msec_ = 2000;
    nhops_abort_ = 3;
    start_ttl_ = 1;
    max_ttl_ = 30;
    ip2AS_mapping_ = NULL;
    stop_enter_AS = -2;
}

template <class ProbePacket>
void PA_Traceroute<ProbePacket>::addOption_StartTTL (int start_ttl)
{
    start_ttl_ = start_ttl;
}

template <class ProbePacket>
void PA_Traceroute<ProbePacket>:: addOption_MaxTTL (int max_ttl)
{
    max_ttl_ = max_ttl;
}

template <class ProbePacket>
void PA_Traceroute<ProbePacket>:: addOption_Timeout (int timeout_msec)
{
    timeout_msec_ = timeout_msec;
}

template <class ProbePacket>
void PA_Traceroute<ProbePacket>:: addOption_nProbe_per_Hop (int nprobe_per_hop)
{
    nprobe_per_hop_ = nprobe_per_hop;
}

template <class ProbePacket>
void PA_Traceroute<ProbePacket>:: addOption_nMaxGoodProbe_per_Hop (int nMaxGoodProbe_per_hop)
{
    nMaxGoodProbe_per_hop_ = nMaxGoodProbe_per_hop;
}

template <class ProbePacket>
void PA_Traceroute<ProbePacket>:: addOption_Enter_Stop_AS (int stop_AS)
{
    ASSERT (stop_AS != -2, "stop_AS cannot use the magic number -2");
    stop_enter_AS = stop_AS;
}

template <class ProbePacket>
void PA_Traceroute<ProbePacket>:: addOption_Consecu_Noresp_Abort (int nhops_abort)
{
    nhops_abort_ = nhops_abort;
}

template <class ProbePacket>
void PA_Traceroute<ProbePacket>:: addOption_IP2AS_Mapping (patricia_tree_t* ip2AS_mapping)
{
    ip2AS_mapping_ = ip2AS_mapping;
}


template <class ProbePacket>
PA_Traceroute<ProbePacket>::
~PA_Traceroute ()
{
}


template <class ProbePacket>
bool
PA_Traceroute<ProbePacket>::
check_stop_max_ttl (string* reason_to_stop, uint8 ttl)
{
    if (ttl > max_ttl_) {
        *reason_to_stop = "maxhop";
        return true;
    }
    return false;
}

template <class ProbePacket>
bool
PA_Traceroute<ProbePacket>::
check_stop_enter_AS (string* reason_to_stop, int ASN)
{
    if (ASN == stop_enter_AS) {
        *reason_to_stop = "enter-AS";
        return true;
    }
    return false;
}

template <class ProbePacket>
bool
PA_Traceroute<ProbePacket>::
check_dest_reached (string* reason_to_stop, struct TTL_Reply_Detail* reply_detail)
{
    if (reply_detail->logic_code != TTL_Reply_Detail::ON_THE_WAY) {
        *reason_to_stop = reply_detail->net_code;
        return true;
    }
    return false;
}

template <class ProbePacket>
bool
PA_Traceroute<ProbePacket>::
check_stop_consecu_noresp (string* reason_to_stop, int consecutive_noresp)
{
    if (consecutive_noresp >= nhops_abort_) {
        *reason_to_stop = "*";
        return true;
    }
    return false;
}


#define MAX_PACKET_LEN 1500

template <class ProbePacket>
void
PA_Traceroute<ProbePacket>::
run (void)
{
    bool stop_algo = false;
    int consecutive_noresp = 0;
    unsigned short current_pkt_id = init_pkt_id_;//The one going to be sent or just sent

    log (INFO, "run, start_ttl = %u max_ttl = %u", start_ttl_, max_ttl_);
    
    for (int i = 0; ! stop_algo; i ++) {
        if (stop_algo = check_stop_max_ttl (&reason_to_stop, start_ttl_ + i))
            continue;

        log (INFO, "Work on hop %d ttl %d", i, start_ttl_ + i);
        
        Hop_Record empty_hop;
        records_.push_back (empty_hop);      

        bool no_resp = true;
        int j, ngood_probes;
        for (j = 0, ngood_probes = 0; 
             j < nprobe_per_hop_ && (nMaxGoodProbe_per_hop_ <= 0 || ngood_probes < nMaxGoodProbe_per_hop_); 
             j++)
        {
            ASSERT(i == records_.size() - 1, 
                "invalid i %d, size %d\n", i, records_.size());

            Per_Probe_Record pr;
            pr.from_ip = 0;
            pr.rtt_msec = -1;
            records_[i].tries.push_back (pr);

            uint8 ttl = i + start_ttl_;
            probep_->send (current_pkt_id, ttl);
            gettimeofday (&(records_[i].tries[j].tm_sent), NULL);

            struct timeval tm_recv;
            //Process the reply
            TTL_Reply_Detail reply_detail;
            if (wait_for_reply (timeout_msec_, current_pkt_id, &tm_recv, &reply_detail) == 0) {
                ASSERT(i == records_.size() - 1, 
                    "invalid i %d, size %d\n", i, records_.size());

                ASSERT(j == records_[i].tries.size() - 1, 
                    "invalid i %d, size %d\n", j, 
                    records_[i].tries.size() );

                Per_Probe_Record &cur_ent = records_[i].tries[j];
                double raw_rtt = timeval_diff (tm_recv, cur_ent.tm_sent) / 1000.0;
                cur_ent.rtt_msec = (raw_rtt >= 0 ? raw_rtt : 0); // - proc_delay;
                cur_ent.from_ip = reply_detail.from_ip;
                cur_ent.return_code = reply_detail.net_code;
                
                post_process_reply (&cur_ent);

                if (! stop_algo) stop_algo = check_stop_enter_AS (&reason_to_stop, cur_ent.ASN);

                if (! stop_algo) stop_algo = check_dest_reached (&reason_to_stop, &reply_detail);

                //printf ("i=%d j=%d stop_algo = %d\n", i,j, stop_algo);
                
                no_resp = false;
                ngood_probes ++;
                log(INFO, "rtt = %.2f", cur_ent.rtt_msec);
            }
            
            log(INFO, "Wait done");

            current_pkt_id ++;
            if (current_pkt_id >= max_pkt_id_) current_pkt_id = init_pkt_id_;
        }

        if (no_resp) consecutive_noresp ++;
        else consecutive_noresp = 0;

        if (! stop_algo) stop_algo = check_stop_consecu_noresp (&reason_to_stop, consecutive_noresp);
    }
}


template <class ProbePacket>
int
PA_Traceroute<ProbePacket>::
wait_for_reply (int timeout_msec, uint16 expected_seq, struct timeval *tm_recv,
    TTL_Reply_Detail *detail)
{
    struct timeval start;
    gettimeofday (&start, NULL);

    while (1) {
        struct timeval now;
        gettimeofday (&now, NULL);
        int rem_timeout_msec = timeout_msec - timeval_diff (now, start) / 1000;
        if (rem_timeout_msec <= 0) {
            return 1; //timeout
        }
        
        char buffer[MAX_PACKET_LEN];
        int packlen = MAX_PACKET_LEN;
        log (INFO, "Call recv with timeout = %d msec", rem_timeout_msec);
        
        if (pkt_clnt_->recv(buffer, &packlen, tm_recv, rem_timeout_msec)) {
            uint16 seq;
            bool ret = probep_->parse (buffer, packlen, &seq, detail);

            if (ret) {
                log(INFO, "parse, seq = %u", seq);
                
                if (seq == expected_seq) {
                    log(INFO, "got reply");
                    return 0;
                }
                else
                    log(INFO, "bad packet id");
            }
            else
                log(INFO, "bad packet");
        }
        else return 1;//timeout
    }
}


template <class ProbePacket>
void
PA_Traceroute<ProbePacket>::
post_process_reply (struct PA_Traceroute<ProbePacket>::Per_Probe_Record* cur_ent)
{
    cur_ent->ASN = 0;
    if (ip2AS_mapping_) {
        patricia_node_t* node = NULL;
        char temp[32];
        strcpy (temp, NetUtil::ip_uint32_to_str(cur_ent->from_ip).c_str());
        node = try_search_best(ip2AS_mapping_, temp);
        
        if (node) {
            cur_ent->ASN = (int)node->user1;
            log (INFO, "ASN = %d", cur_ent->ASN);
        }
    }
}


template <class ProbePacket>
string
PA_Traceroute<ProbePacket>::
output (void)
{
    ostringstream oss (ostringstream::out);

    struct timeval tm;
    gettimeofday (&tm, NULL);

    oss << probep_->info_str () << " " << reason_to_stop << " finish_time= " << tm.tv_sec << endl;
    
    for (int i = 0; i < records_.size(); i ++) {
        oss << i;
        for (int j = 0; j < records_[i].tries.size(); j ++) {
            if (records_[i].tries[j].rtt_msec > 0) {
                oss << " " << NetUtil::ip_uint32_to_str (records_[i].tries[j].from_ip) << 
                    " [" << records_[i].tries[j].rtt_msec << " ms]";
                if (ip2AS_mapping_)
                    oss << " {" << records_[i].tries[j].ASN << "}";
                if (records_[i].tries[j].return_code != "")
                    oss << " " << records_[i].tries[j].return_code;
            }                
            else {
                oss << " * [ ]";
                if (ip2AS_mapping_) { oss << " { }"; }
            }
        }
        oss << endl;
    }

    return oss.str();
}


#endif                                            //__PA_TRACEROUTE_H__

