/*************************************************************/
/* iTraceroute (part of iSPY) Project                        */
/* Copyright (c) 2008, Zheng Zhang, Purdue University        */
/*************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <arpa/inet.h>
#include <string.h>
#include <pthread.h>

#include <string>
using namespace std;

#include "NetUtil.h"
#include "Common.h"
#include "PacketCatcher.h"

struct Option {
    int protocol;
    unsigned short port;
    int debug;
};

struct Option opt;

void usage (void)
{
    printf ("packetd -d debug_level protocol(tcp|udp|icmp) port(can be 0)\n");
}

void init_option (void)
{
    opt.debug = WARN;
    opt.port = 0;
    opt.protocol = IPPROTO_TCP;
}

void parse_cmd (int argc, char *argv[])
{
    int c;
  
    while ((c = getopt(argc, argv, "d:")) > 0) {
        switch(c) {
        case 'd':
            opt.debug = atoi(optarg);
            break;
        default:
            usage(); exit (1);
        }
    }
    
    argc = argc - optind + 1;
    argv = argv + optind - 1;
    
    
    if (argc != 3) {
        usage();
        exit(1);
    }
    
    if (!strcmp (argv[1], "tcp"))
        opt.protocol = IPPROTO_TCP;
    else if (!strcmp (argv[1], "udp"))
        opt.protocol = IPPROTO_UDP;
    else if (!strcmp (argv[1], "icmp"))
        opt.protocol = IPPROTO_ICMP;
    else {
        usage(); exit (1);
    }
    
    opt.port = atoi(argv[2]);
    
    set_log_level (opt.debug);
}


int main (int argc, char** argv)
{
    init_option ();
    parse_cmd (argc, argv);

    IncomPktMgr pc (opt.protocol);
    IncomPktMgr_Client pkt_clnt;
    pc.addClient (&pkt_clnt, 0);
    IncomPktMgr_Rule_Default default_rule;
    pc.append_Rule (&default_rule);
    pc.asyn_Run ();

    while (1) {
        char buffer[1500];
        int len = 1500;
        struct timeval tm;
        if (pkt_clnt.recv (buffer, &len, NULL, 2000)) {
            printf ("len=%d packet_content=%x... time=%lu:%lu\n", len, buffer,
                    tm.tv_sec, tm.tv_usec);
            if (len >= sizeof (struct _iphdr)) {
                struct _iphdr* ip1 = (struct _iphdr*) buffer;
                printf ("%s\n", NetUtil::print_ip (ip1).c_str());
            }
        }
    }
    return 0;
}


