#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "Common.h"

struct Option {
    int test_num;
    int debug;
};

Option opt;

void test_timeval (void)
{
        struct timeval tm1, tm2;
        tm1.tv_sec = 100;
        tm1.tv_usec = 945020;
        tm2.tv_sec = 101;
        tm2.tv_usec = 45020;

        int gap_nsec = timeval_diff (tm2, tm1);
        printf ("diff = %d nsec\n", gap_nsec);

        struct timeval tm3, tm4;
        tm3 = timeval_incr (tm1, gap_nsec);
        tm4 = timeval_incr (tm2, -gap_nsec);
        printf ("tm3 %u:%u, tm4 %u:%u\n",
            tm3.tv_sec, tm3.tv_usec, tm4.tv_sec, tm4.tv_usec);

        ASSERT (tm3.tv_sec == tm2.tv_sec &&
            tm3.tv_usec == tm2.tv_usec, "[1]");
        ASSERT (tm4.tv_sec == tm1.tv_sec &&
            tm4.tv_usec == tm1.tv_usec, "[1]");
}

void testcase(int case_no)
{
    switch(case_no)
    {
        case 0:
            for(int i = 1; i <= 1; i ++)
                testcase(i);
            break;
        case 1:
            test_timeval();
            break;
        default:
            ASSERT(0, "Wrong testcase number\n");
    }
    printf ("Test case %d PASSED !!!!!\n", case_no);
}

void usage (void)
{
    printf ("test -d debug <case_number>\n");
}

void init_option (void)
{
    opt.debug = WARN;
    opt.test_num = 0;
}

void parse_cmd (int argc, char *argv[])
{
    int c;
  
    init_option ();
    
    while ((c = getopt(argc, argv, "d:")) > 0) {
        switch(c) {
        case 'd':
            opt.debug = atoi(optarg);
            break;
        default:
            usage(); exit (1);
        }
    }
    
    argc = argc - optind + 1;
    argv = argv + optind - 1;
    
    if (argc != 2) {
        usage();
        exit(1);
    }
    
    opt.test_num = atoi(argv[1]);
    set_log_level (opt.debug);
}


int main (int argc, char** argv)
{
    init_option ();
    parse_cmd (argc, argv);

    testcase (opt.test_num);
    return 0;
}
